using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Net.WebSockets;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using UnityEngine;

namespace EventGames
{
    public class WebSocketCommunicator : ICommunicatable
    {
        private enum State
        {
            None,
            EstablishingConnection,
            WaitingForConnectedCallback,
            Running
        }
        private State state;

        private static int NextID;
        private string id;

        private ClientWebSocket webSocket;
        private UTF8Encoding encoder;
        private Uri serverUri;

        private Thread receiverThread;
        private Thread senderThread;
        private Queue<string> receiveQueue;
        private Queue<string> sendQueue;

        private Action connectedCallback;
        private Action<string> messageReceivedCallback;

        //interface
        public void Open(Action connectedCallback, Action<string> messageReceivedCallback)
        {
            state = State.EstablishingConnection;
            this.connectedCallback = connectedCallback;
            this.messageReceivedCallback = messageReceivedCallback;

            id = "WS-" + NextID++;

            receiveQueue = new Queue<string>();
            sendQueue = new Queue<string>();

            receiverThread = new Thread(ReceiverThread);
            senderThread = new Thread(SenderThread);
            webSocket = new ClientWebSocket();
            encoder = new UTF8Encoding();
            serverUri = new Uri($"wss://{Settings.I.StandaloneWebSocketServer}");

            //connect to server
            new Thread(async () =>
            {
                await webSocket.ConnectAsync(serverUri, CancellationToken.None);
                while (webSocket.State == WebSocketState.Connecting)
                {
                    if (Settings.I.Debug)
                    {
                        Debug.Log($"{id}: Connecting to WebSocket {serverUri}");
                    }

                    Task.Delay(50).Wait();
                }

                if (webSocket.State == WebSocketState.Open)
                {
                    if (Settings.I.Debug)
                    {
                        Debug.Log($"{id}: Connected to WebSocket");
                    }

                    receiverThread.Start();
                    senderThread.Start();

                    state = State.WaitingForConnectedCallback;
                }
                else
                {
                    if (Settings.I.Debug)
                    {
                        Debug.Log($"{id}: WebSocket status {webSocket.State}");
                    }
                }
            }).Start();
        }
        public void Update()
        {
            if (state == State.WaitingForConnectedCallback)
            {
                connectedCallback?.Invoke();
                state = State.Running;
            }
            else if (state == State.Running)
            {
                while (receiveQueue.Count > 0)
                {
                    messageReceivedCallback(receiveQueue.Dequeue());
                }
            }
        }
        public void Close()
        {
            webSocket?.CloseAsync(WebSocketCloseStatus.NormalClosure, "", CancellationToken.None);
            receiverThread?.Abort();
            senderThread?.Abort();
        }
        public void Send(string message)
        {
            sendQueue.Enqueue(message);
        }

        //threads
        private void SenderThread()
        {
            if (Settings.I.Debug)
            {
                Debug.Log($"{id}: Started sender thread");
            }

            string message;
            while (true)
            {
                if (webSocket.State == WebSocketState.Open)
                {
                    while (sendQueue.Count > 0)
                    {
                        message = sendQueue.Dequeue();
                        webSocket.SendAsync(new ArraySegment<byte>(encoder.GetBytes(message)), WebSocketMessageType.Text, true, CancellationToken.None).Wait();

                        if (Settings.I.Debug)
                        {
                            Debug.Log($"{id}: Send message '{message}'");
                        }
                    }
                }
            }
        }
        private async void ReceiverThread()
        {
            if (Settings.I.Debug)
            {
                Debug.Log($"{id}: Started receiver thread ");
            }

            WebSocketReceiveResult chunk;
            ArraySegment<byte> buffer;
            MemoryStream memoryStream;
            string message;

            while (true)
            {
                if (webSocket.State == WebSocketState.Open)
                {
                    //clear buffers
                    buffer = new ArraySegment<byte>(new byte[4 * 1024]);
                    memoryStream = new MemoryStream();

                    //read message from web socket
                    do
                    {
                        chunk = await webSocket.ReceiveAsync(buffer, CancellationToken.None);
                        memoryStream.Write(buffer.Array, buffer.Offset, chunk.Count);
                    } while (!chunk.EndOfMessage);

                    //if the message is of type text
                    if (chunk.MessageType == WebSocketMessageType.Text)
                    {
                        //convert message into string
                        memoryStream.Seek(0, SeekOrigin.Begin);
                        using (var reader = new StreamReader(memoryStream, Encoding.UTF8))
                        {
                            message = reader.ReadToEnd();
                        }

                        //added message to queue
                        if (!string.IsNullOrEmpty(message))
                        {
                            receiveQueue.Enqueue(message);

                            if (Settings.I.Debug)
                            {
                                Debug.Log($"{id}: Received '{message}'");
                            }
                        }
                        else
                        {
                            if (Settings.I.Debug)
                            {
                                if (message == null)
                                {
                                    Debug.Log($"{id}: Received a empty message");
                                }
                                else
                                {
                                    Debug.Log($"{id}: Received a empty message");
                                }
                            }
                        }
                    }
                    else
                    {
                        if (Settings.I.Debug)
                        {
                            Debug.Log($"{id}: Received a none text response");
                        }
                    }
                }
            }
        }
    }
}