using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using UnityEngine;
using UnityEngine.Events;

namespace EventGames
{
    public class RuntimeManager : MonoBehaviour
    {
        public const string gameObjectName = "_EventGames Runtime Manager";
        public static bool RunningInTest;

        private static RuntimeManager _i;
        public static RuntimeManager I
        {
            get
            {
                if (_i == null)
                {
                    _i = new GameObject(gameObjectName).AddComponent<RuntimeManager>();
                }
                return _i;
            }
        }

        private ICommunicatable communicator;
        public readonly UnityEvent<string, string> onServerReady = new UnityEvent<string, string>();
        public readonly UnityEvent<int, string> onPlayerInput = new UnityEvent<int, string>();
        public readonly UnityEvent<int, int, string> onPlayerJoin = new UnityEvent<int, int, string>();
        public readonly UnityEvent<int> onPlayerLeave = new UnityEvent<int>();

        public void Send(string message)
        {
            communicator.Send(message);
        }

        private void Start()
        {
            if (RunningInTest)
            {
                return;
            }

            switch (Settings.I.Communicator)
            {
                case Settings.Communicators.WebSocket:
                    communicator = new WebSocketCommunicator();
                    break;
                case Settings.Communicators.JavaScript:
                    communicator = new JavaScriptCommunicator();
                    break;
            }

            if (communicator == null)
            {
                throw new Exception("No communicator was fund");
            }

            communicator?.Open(OnConnected, OnMessageReceived);
        }
        private void Update()
        {
            communicator?.Update();
        }
        private void OnDestroy()
        {
            communicator?.Close();
        }

        private void OnConnected()
        {
            string runtime = $"Unity: {Application.unityVersion} {Application.platform} - Game: {Application.version}";
            string screenId = Settings.I.ScreenId;
            communicator.Send(new AcquireTunnelMessage(screenId, runtime).ToJson());
        }
        private void OnMessageReceived(string json)
        {
            MessageBase messageBase = MessageBase.FromJson(json);

            //return pong timestamp
            if (messageBase.tunnel_timestamp != 0)
            {
                communicator.Send(new PongMessage(messageBase.tunnel_timestamp).ToJson());
            }

            if (messageBase.command == "tunnel_configured")
            {
                TunnelConfiguredMessage message = TunnelConfiguredMessage.FromJson(json);
                onServerReady?.Invoke(message.tunnel_key, message.key_server_url);

                string joystickSetup = Regex.Replace(Settings.I.JoystickSetup.text, @"\t|\r", "");
                string joystickLocalized = Regex.Replace(Settings.I.JoystickLocalized.text, @"\t|\r", "");
                communicator.Send(new JoystickServerMessage(joystickSetup, joystickLocalized).ToJson());
            }
            else if (messageBase.command == "join")
            {
                PlayerJoinMessage message = PlayerJoinMessage.FromJson(json);

                //send joystick
                communicator.Send(new JoystickClientSetupMessage(message.joystick_id).ToJson());

                //activate joystick
                string color = "";
                string colorName = "";
                if (message.tint_index < Settings.I.Colors.Length)
                {
                    color = Settings.I.Colors[message.tint_index];
                    colorName = Settings.I.ColorNames[message.tint_index];
                }
                communicator.Send(new JoystickClientActivateMessage(message.joystick_id, colorName, color).ToJson());

                //join game
                onPlayerJoin?.Invoke(message.joystick_id, message.tint_index, message.name);
            }
            else if (messageBase.command == "leave")
            {
                PlayerLeaveMessage message = PlayerLeaveMessage.FromJson(json);
                onPlayerLeave?.Invoke(message.joystick_id);
            }
            else
            {
                PlayerJoinMessage message = PlayerJoinMessage.FromJson(json);
                if (message != null && message.joystick_id != -1)
                {
                    onPlayerInput?.Invoke(message.joystick_id, json);
                }
                else
                {
                    Debug.LogError($"Unable to handle received message: {json}");
                }
            }
        }
    }
}