using UnityEngine;
using Spine.Unity;
using EventGames;
using System.Collections;
using UnityEngine.Serialization;

namespace AlienRunner
{
    public class PlayerAnimation : MonoBehaviour, IPlayerSetupable
    {
        private enum Track
        {
            Base = 0,
            Crown = 1,
            Aim = 2,
            Shield = 3,
            Poop = 4
        }

        [SerializeField]
        private SkeletonAnimation skeletonAnimation;

        [Header("Movement")]
        [SerializeField]
        private AnimationReferenceAsset fallIdle;
        [SerializeField]
        private AnimationReferenceAsset flyIdle;
        [SerializeField]
        private AnimationReferenceAsset running;

        [Header("Crown")]
        [SerializeField]
        private AnimationReferenceAsset crown;
        [SerializeField]
        private AnimationReferenceAsset noCrown;
        [SerializeField]
        [FormerlySerializedAs("crowColors")]
        private Color[] crownColors;

        [Header("shield")]
        [SerializeField]
        private AnimationReferenceAsset shieldOn;
        [SerializeField]
        private AnimationReferenceAsset shieldOff;

        [Header("Other")]
        [SerializeField]
        private AnimationReferenceAsset stunned;
        [SerializeField]
        private AnimationReferenceAsset aim;
        [SerializeField]
        private AnimationReferenceAsset pooped;

        private PlayerState playerState;

        public void Setup(int id)
        {
            playerState = GetComponent<PlayerState>();
            SetBaseAnimation(flyIdle);

            //color
            Color color = Players.I.GetColor(gameObject);
            foreach (var slot in skeletonAnimation.skeleton.Slots)
            {
                if (slot.ToString().Contains("tint_1"))
                    slot.SetColor(color);
            }

            skeletonAnimation.state.SetAnimation((int)Track.Shield, shieldOff, false).TimeScale = 99f;
            Crown(0, 0);

            playerState.onEnter += OnStateEnter;
            playerState.onExit += OnStateExit;
        }
        private void OnDestroy()
        {
            playerState.onEnter -= OnStateEnter;
            playerState.onExit -= OnStateExit;
        }

        private void Update()
        {
            float targetRotation = 0;
            if (playerState.IsMoving)
            {
                if (playerState.IsPooped)
                {

                }
                else if (playerState.IsSpeedBoosting)
                {
                    SetBaseAnimation(flyIdle);
                    targetRotation = 45;
                }
                else if (playerState.IsRunning)
                {
                    SetBaseAnimation(running, 4f);
                }
                else if (playerState.IsFlying)
                {
                    SetBaseAnimation(flyIdle, 3);
                }
                else
                {
                    SetBaseAnimation(fallIdle);
                }
            }
            else
            {
                if (playerState.IsStunned)
                {
                    SetBaseAnimation(stunned, 3, true);
                }
                if (playerState.IsGameEnd)
                {
                    SetBaseAnimation(flyIdle, 3, true);
                }
            }

            //rotate
            float angle = Mathf.MoveTowardsAngle(skeletonAnimation.transform.eulerAngles.z, -targetRotation, 180 * Time.deltaTime);
            skeletonAnimation.transform.eulerAngles = new Vector3(0, 0, angle);
        }

        private void OnStateExit()
        {
            if (playerState.IsPooped)
            {
                Pooped(false);
            }
            if (playerState.IsPooped)
            {
                Pooped(false);
            }
        }
        private void OnStateEnter()
        {
            if (playerState.IsPooped)
            {
                Pooped(true);
            }
        }

        private void SetBaseAnimation(AnimationReferenceAsset animation, float timeScale = 1f, bool loop = true)
        {
            if (skeletonAnimation.AnimationName == animation.name)
            {
                return;
            }
            skeletonAnimation.state.SetAnimation((int)Track.Base, animation, loop).TimeScale = timeScale;
        }
        
        private void Pooped(bool active)
        {
            if (active)
            {
                skeletonAnimation.state.SetAnimation((int)Track.Base, pooped, false).TimeScale = 2f;
                skeletonAnimation.state.SetAnimation((int)Track.Poop, pooped, false).TimeScale = 2f;
            }
            else
            {
                skeletonAnimation.state.SetEmptyAnimation((int)Track.Poop, 0);
            }
        }
        
        public void Crown(int placement, int score)
        {
            if (placement < crownColors.Length && score != 0)
            {
                skeletonAnimation.state.SetAnimation((int)Track.Crown, crown, true);

                foreach (var slot in skeletonAnimation.skeleton.Slots)
                {
                    if (slot.ToString().Contains("spaceman/crown"))
                        slot.SetColor(crownColors[placement]);
                }
            }
            else
            {
                skeletonAnimation.state.SetEmptyAnimation((int)Track.Crown, 0);
            }
        }
        
        public void Aim(bool active)
        {
            if (active)
            {
                skeletonAnimation.state.SetAnimation((int)Track.Aim, aim, false).TimeScale = 2f;
            }
            else
            {
                skeletonAnimation.state.SetEmptyAnimation((int)Track.Aim, 0);
            }
        }
        
        public void Shield(bool active)
        {
            StartCoroutine(ShieldIE(active));
        }
        public IEnumerator ShieldIE(bool active)
        {
            Spine.TrackEntry entry;
            if (active)
            {
                entry = skeletonAnimation.state.SetAnimation((int)Track.Shield, shieldOn, false);
            }
            else
            {
                entry = skeletonAnimation.state.SetAnimation((int)Track.Shield, shieldOff, false);
                entry.TimeScale = 4;
            }
            yield return new WaitForSpineAnimationComplete(entry);
        }
    }
}
