using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class PlayerMovement : MonoBehaviour, IPlayerSetupable
    {
        [SerializeField]
        private MeshRenderer meshRenderer;
        [SerializeField]
        private float upwardsSpeed;
        [SerializeField]
        private float gravityScale;
        [SerializeField]
        private float idealPosAdditionalMoveFactor = 0.5f;
        [HideInInspector]
        public float speedBoost = 0;
        [SerializeField]
        private float minPoopVelocity;
        [SerializeField]
        private float maxPoopVelocity;

        [Header("Ground Raycast")]
        [SerializeField]
        private LayerMask groundRaycastLayerMask;
        [SerializeField]
        private float groundRaycastDistance;


        public enum HitType
        {
            Stun,
            Kill
        }
        public new Rigidbody2D rigidbody;
        private PlayerState state;
        private PlayerInput input;

        public event Action OnHit;

        public void Setup(int id)
        {
            input = GetComponent<PlayerInput>();
            state = GetComponent<PlayerState>();
            rigidbody = GetComponent<Rigidbody2D>();
            state.onEnter += OnStateEnter;
            state.onExit += OnStateExit;
        }
        private void OnDestroy()
        {
            state.onEnter -= OnStateEnter;
            state.onExit -= OnStateExit;
        }

        private void UpdateMovement()
        {
            if (state.IsStunned)
            {
                rigidbody.velocity = Vector2.zero;
            }

            if (Players.I != null)
            {
                if (state.IsMoving || state.IsPooped)
                {
                    transform.position += Vector3.right * GameState.I.GameSpeed * (1 + speedBoost) * Time.deltaTime;

                    float idealX = Players.I.GetIdealX(gameObject);
                    Vector2 idealPosition = new Vector2(idealX, rigidbody.position.y);
                    transform.position = Vector2.MoveTowards(transform.position, idealPosition, GameState.I.GameSpeed * Time.deltaTime * idealPosAdditionalMoveFactor);
                }


                if (state.IsMoving && input.Controller.Fly())
                {
                    rigidbody.velocity = Vector2.up * upwardsSpeed;
                }
            }
        }

        private void Update()
        {
            UpdateMovement();

            if (state.IsMoving)
            {
                if (!meshRenderer.IsVisibleMainCamera() && rigidbody.position.x < Camera.main.ViewportToWorldPoint(Vector3.zero).x)
                {
                    state.Set(PlayerState.State.Dead);
                }

                if (!state.IsSpeedBoosting)
                {
                    if (input.Controller.Fly())
                    {
                        if (!state.IsFlying)
                        {
                            state.Set(PlayerState.State.Flying);
                        }
                    }
                    else
                    {
                        RaycastHit2D hit = Physics2D.Raycast(transform.position, Vector2.down, groundRaycastDistance, groundRaycastLayerMask);
                        if (hit.collider != null)
                        {
                            Debug.DrawLine(transform.position, transform.position + Vector3.down * groundRaycastDistance, Color.red);
                            if (!state.IsRunning)
                            {
                                state.Set(PlayerState.State.Running);
                            }
                        }
                        else
                        {
                            Debug.DrawLine(transform.position, transform.position + Vector3.down * groundRaycastDistance, Color.white);

                            if (rigidbody.velocity.y < 0)
                            {
                                if (!state.IsFalling)
                                {
                                    state.Set(PlayerState.State.Falling);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void OnStateEnter()
        {
            if (state.IsStunned)
            {
                SetGravityEnabled(false);
            }
            if (state.IsDead)
            {
                SetPhysicsSimulationEnabled(false);
                SetPositionAndResetVelocity(Vector3.one * -9999);
            }
            if (state.IsMoving)
            {
                SetGravityEnabled(true);
            }
            if (state.IsEaten)
            {
                SetPhysicsSimulationEnabled(false);
            }
            if(state.IsPooped)
            {
                rigidbody.velocity = new Vector2(rigidbody.velocity.x, UnityEngine.Random.Range(minPoopVelocity, maxPoopVelocity));
            }
            if(state.IsGameEnd)
            {
                SetPhysicsSimulationEnabled(false);
            }
        }
        private void OnStateExit()
        {
            if (state.IsStunned)
            {
                SetGravityEnabled(true);
            }
            if (state.IsDead)
            {
                SetPhysicsSimulationEnabled(true);
                SetPositionAndResetVelocity(Players.I.GetRespawnPosition());
            }
            if (state.IsEaten)
            {
                SetPhysicsSimulationEnabled(true);
            }
        }

        /// <summary>
        /// Enable or disable gravity.
        /// </summary>
        /// <param name="gravityEnabled">True => gravityScale = 1 False => gravityScale = 0</param>
        public void SetGravityEnabled(bool gravityEnabled)
        {
            rigidbody.gravityScale = gravityEnabled ? gravityScale : 0;
        }

        public void SetPhysicsSimulationEnabled(bool simulationEnabled)
        {
            rigidbody.simulated = simulationEnabled;
        }
        public void SetPositionAndResetVelocity(Vector3 pos)
        {
            rigidbody.position = pos;
            transform.position = pos;
            rigidbody.velocity = Vector2.zero;
        }

        public void Hit(HitType hitType)
        {
            if (state.IsEaten || state.IsPooped)
                return;
            if (state.IsStunned)
                return;
            if (state.IsInvincible)
                return;
            //If a player is shielded, we signal that the player was hit, but nothing happens to them
            if (state.IsShielded)
            {
                OnHit?.Invoke();
                return;
            }

            //Otherwise they're either stunned or killed depending on the type of object they were hit by
            if (hitType == HitType.Stun)
            {
                state.Set(PlayerState.State.Stunned);
            }
            else if (hitType == HitType.Kill)
            {
                state.Set(PlayerState.State.Dead);
            }
            OnHit?.Invoke();
        }
    }
}
