using System;
using UnityEngine;

namespace AlienRunner
{
    public class PlayerState : MonoBehaviour, IPlayerSetupable
    {
        public enum State
        {
            Waiting,
            Running,
            Falling,
            Flying,
            Stunned,
            Dead,
            SpeedBoost,
            Eaten,
            Pooped,
            GameEnd,
        }

        [Header("Timers")]
        [SerializeField]
        private float stunTime;
        private float stunTimer;

        [SerializeField]
        private float respawnTime;
        private float respawnTimer;
        public float RespawnTime => respawnTime;

        [SerializeField]
        private float invincibleTime;
        private float invincibleTimer;
        public float InvincibleTime => invincibleTime;

        [SerializeField]
        private float poopTime;
        private float poopTimer;
        public float PoopTime => poopTime;

        public event Action onEnter;
        public event Action onExit;

        private State? Current = null;
        public void Set(State newState)
        {
            if(Current == newState)
            {
                return;
            }
            onExit?.Invoke();
            Current = newState;
            onEnter?.Invoke();
        }

        public bool IsMoving
        {
            get
            {
                return Current == State.Running ||
                       Current == State.Falling ||
                       Current == State.Flying ||
                       Current == State.SpeedBoost;
            }
        }
        public bool IsFlying
        {
            get
            {
                return Current == State.Flying;
            }
        }
        public bool IsFalling
        {
            get
            {
                return Current == State.Falling;
            }
        }
        public bool IsRunning
        {
            get
            {
                return Current == State.Running;
            }
        }
        public bool IsStunned
        {
            get
            {
                return Current == State.Stunned;
            }
        }
        public bool IsSpeedBoosting
        {
            get
            {
                return Current == State.SpeedBoost;
            }
        }
        public bool IsDead
        {
            get
            {
                return Current == State.Dead;
            }
        }
        public bool IsInvincible
        {
            get
            {
                return invincibleTimer > 0;
            }
        }

        public bool IsShielded 
        {
            get
            {
                return GetComponentInChildren<ShieldPowerUp>() != null;
            }
        }

        public bool IsWaiting => Current == State.Waiting;
        public bool IsEaten => Current == State.Eaten;
        public bool IsPooped => Current == State.Pooped;
        public bool IsGameEnd => Current == State.GameEnd;


        public void Setup(int id)
        {
            onEnter += OnStateEnter;
            onExit += OnStateExit;
            GameState.I.OnGameStart += () =>
            {
                Set(State.Falling);
            };
            GameState.I.OnGameEnd += () =>
            {
                Set(State.GameEnd);
            };
        }

        private void OnDestroy()
        {
            onEnter -= OnStateEnter;
            onExit -= OnStateExit;
        }
        private void Update()
        {
            if (IsStunned)
            {
                stunTimer -= Time.deltaTime;
                if (stunTimer <= 0)
                {
                    Set(State.Falling);
                }
            }
            if (IsDead)
            {
                respawnTimer -= Time.deltaTime;
                if (respawnTimer <= 0)
                {
                    Set(State.Falling);
                }
            }
            if (IsMoving)
            {
                if (invincibleTimer > 0)
                {
                    invincibleTimer -= Time.deltaTime;
                }
            }
            if(IsPooped)
            {
                poopTimer -= Time.deltaTime;
                if(poopTimer < 0)
                {
                    Set(State.Falling);
                }
            }
        }

        private void OnStateEnter()
        {
            if (IsStunned)
            {
                stunTimer = stunTime;
            }
            if (IsDead)
            {
                respawnTimer = respawnTime;
            }
            if(IsPooped)
            {
                poopTimer = poopTime;
            }
        }
        private void OnStateExit()
        {
            if (IsDead)
            {
                MakeInvincible();
            }
        }

        public void MakeInvincible()
        {
            invincibleTimer = invincibleTime;
        }
    }
}