using EventGames;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace AlienRunner
{
    public class Players : Singleton<Players>
    {
        public const int max = 10;

        protected override bool InstantiateIfMissing => false;
        protected override bool UseDontDestroyOnLoad => false;

        [SerializeField]
        private GameObject playerPrefab;
        [SerializeField]
        private float offset;
        [SerializeField]
        private float gap;

        private readonly List<GameObject> players = new List<GameObject>();
        private readonly Dictionary<GameObject, int> ids = new Dictionary<GameObject, int>();
        private readonly Dictionary<GameObject, int> ranks = new Dictionary<GameObject, int>();
        private readonly Dictionary<GameObject, int> tintIndexs = new Dictionary<GameObject, int>();
        private readonly Dictionary<GameObject, string> names = new Dictionary<GameObject, string>();
        private readonly Dictionary<GameObject, Color> colors = new Dictionary<GameObject, Color>();
        private readonly Dictionary<GameObject, PlayerScore> scores = new Dictionary<GameObject, PlayerScore>();

        public List<GameObject> AllPlayers => players;
        public List<PlayerScore> OrderedScores => Scores.OrderByDescending(o => o.Value.Score).Select(o => o.Value).ToList();
        public int PlayerCount => players.Count;
        public List<GameObject> GameObjects => players.ToList();
        public Dictionary<GameObject, PlayerScore> Scores => new Dictionary<GameObject, PlayerScore>(scores);

        public int GetId(GameObject player)
        {
            return ids[player];
        }
        public int GetRank(GameObject player)
        {
            return ranks[player];
        }
        public int GetTintIndex(GameObject player)
        {
            return tintIndexs[player];
        }
        public Color GetColor(GameObject player)
        {
            return colors[player];
        }
        public string GetName(GameObject player)
        {
            return names[player];
        }
        public string GetColorName(GameObject player)
        {
            int index = GetTintIndex(player);
            return Settings.I.ColorNames[index];
        }

        private void Update()
        {
            if (GameState.I.CurrentState == GameState.State.Running)
            {
                UpdateRank();
            }
        }

        public GameObject AddPlayer(int id, int tintIndex, string name)
        {
            GameObject player = Instantiate(playerPrefab);

            players.Add(player);
            ids.Add(player, id);
            ranks.Add(player, 0);
            names.Add(player, name);
            tintIndexs.Add(player, tintIndex);
            ColorUtility.TryParseHtmlString("#" + Settings.I.Colors[tintIndex], out Color _color);
            colors.Add(player, _color);
            scores.Add(player, player.GetComponent<PlayerScore>());

            foreach (var com in player.GetComponents<IPlayerSetupable>())
            {
                com.Setup(id);
            }
            player.GetComponent<PlayerState>().Set(PlayerState.State.Waiting);

            player.transform.position = new Vector3(-9999, player.transform.position.y, player.transform.position.y);
            UpdateRank();
            PlayerScore.UpdateAllCrowns();
            if (GameState.I.CurrentState == GameState.State.AwaitingPlayers)
            {
                MoveAllPlayersToIdealX();
            }
            else
            {
                player.GetComponent<PlayerState>().Set(PlayerState.State.Falling);
            }

            return player;
        }
        public void RemovePlayer(int id)
        {
            GameObject gameObject = ids.FirstOrDefault(o => o.Value == id).Key;
            if (gameObject != null)
            {
                players.Remove(gameObject);
                ids.Remove(gameObject);
                ranks.Remove(gameObject);
                names.Remove(gameObject);
                tintIndexs.Remove(gameObject);
                colors.Remove(gameObject);
                scores.Remove(gameObject);
                DestroyImmediate(gameObject);
                UpdateRank();
                PlayerScore.UpdateAllCrowns();
            }
        }

        private void UpdateRank()
        {
            int i = 0;
            foreach (var player in players.OrderBy(o => o.transform.position.x))
            {
                ranks[player] = i;
                i++;
            }
        }
        public float GetIdealX(GameObject player)
        {
            return RankToPosition(ranks[player]);
        }
        private void MoveAllPlayersToIdealX()
        {
            foreach (var player in players)
            {
                player.transform.position = new Vector3(GetIdealX(player), player.transform.position.y, player.transform.position.y);
            }
        }
        public Vector3 GetRespawnPosition()
        {
            return new Vector3(RankToPosition(0) - 0.5f, Camera.main.transform.position.y, 0);
        }
        public float RankToPosition(int rank)
        {
            return Camera.main.ViewportToWorldPoint(Vector3.zero).x + offset + gap * rank;
        }
    }
}
