using System.Collections;
using System.Collections.Generic;
using System.Runtime.Serialization;
using UnityEngine;

namespace AlienRunner
{
    public abstract class Singleton<T> : MonoBehaviour where T : Singleton<T>
    {
        /// <summary>
        /// We cache the destroy frame to avoid instantiating a new singleton on the same frame.
        /// </summary>
        [AllowedStatic]
        private static int destroyFrame = -1;
        public static bool Exists => _i != null;
        [AllowedStatic]
        private static T _i;
        public static T I
        {
            get
            {
                if(_i == null)
                {
                    //Try to find the singleton in the scene first.
                    _i = FindObjectOfType<T>();
                    if(_i == null && UseAutomaticInstantiation)
                    {
                        //Automatically create a gameobject and add this component.
                        if(destroyFrame < Time.frameCount)
                        {
                            _i = new GameObject(typeof(T).Name).AddComponent<T>();
                        }
                    }
                }
                return _i;
            }
        }
        [AllowedStatic]
        private static bool? _useAutomaticInstantiation;
        /// <summary>
        /// If true, this singleton will be automatically added to a new GameObject if it doesn't exist.
        /// </summary>
        private static bool UseAutomaticInstantiation
        {
            get
            {
                if (!_useAutomaticInstantiation.HasValue)
                {
                    //The actual value is implemented as an abstract-bool, on an instance-level
                    //So we need to create an uninitialized instance, to cache the value.
                    _useAutomaticInstantiation = ((T)FormatterServices.GetUninitializedObject(typeof(T))).InstantiateIfMissing;
                }
                return _useAutomaticInstantiation.Value;
            }
        }
        /// <summary>
        /// If true, this singleton will be automatically added to a new GameObject if it doesn't exist.
        /// </summary>
        protected abstract bool InstantiateIfMissing { get; }
        /// <summary>
        /// If true, this gameObject will automatically get DontDestroyOnLoad called (Unless we have a parent)
        /// </summary>
        protected abstract bool UseDontDestroyOnLoad { get; }
        protected virtual void Awake()
        {
            if (this is T t)
            {
                if (_i == null || _i == this)
                {
                    _i = t;
                    if (transform.parent == null && UseDontDestroyOnLoad)
                    {
                        DontDestroyOnLoad(gameObject);
                    }
                }
                else if (_i != this)
                {
                    Destroy(gameObject);
                }
            }
        }
        protected virtual void OnDestroy()
        {
            if (_i == this)
            {
                destroyFrame = Time.frameCount;
                _i = null;
            }
        }
    }
}