using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class PlayerMovementAndInputTests
    {
        [UnityTest]
        public IEnumerator GameState_AwaitingPlayers_NoForwardMovement()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            Vector3 startPos = player.transform.position;

            //-Act
            //GameState should ideally always start on AwaitingPlayers, but in case that changes we're forcing the state here
            GameState.I.SetGameState(GameState.State.AwaitingPlayers);
            yield return new WaitForSeconds(1);

            //-Assert
            //We check if the player is in the same position as they were in the beginning
            Assert.AreEqual(startPos, player.transform.position);
        }

        [UnityTest]
        public IEnumerator GameState_End_NoForwardMovement()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            Vector3 startPos = player.transform.position;

            //-Act
            GameState.I.SetGameState(GameState.State.End);
            yield return new WaitForSeconds(1);

            //-Assert
            //We check if the player is in the same position as they were in the beginning
            Assert.AreEqual(startPos, player.transform.position);
        }

        [UnityTest]
        public IEnumerator GameState_Running_ForwardMovement()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            float startPosX = player.transform.position.x;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            yield return new WaitForSeconds(1);

            //-Assert
            //We check to see if the player's x-position has increased compared to their start position
            float currentX = player.transform.position.x;
            Assert.Greater(currentX, startPosX);
        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator GetInput_StateIsFlying()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();
            PlayerInput input = player.GetComponent<PlayerInput>();

            //-Act
            //Run the game while receiving simulated input
            GameState.I.SetGameState(GameState.State.Running);
            (input.Controller as TestRunnerController).fly = true;
            yield return new WaitForSeconds(1);

            //-Assert
            bool expectedState = true;
            Assert.AreEqual(expectedState, state.IsFlying);

        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator GetInput_PlayerMovesUp()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            float startY = player.transform.position.y;
            PlayerMovement controller = player.GetComponent<PlayerMovement>();
            PlayerInput input = player.GetComponent<PlayerInput>();

            //-Act
            //Run the game while receiving simulated input
            GameState.I.SetGameState(GameState.State.Running);
            (input.Controller as TestRunnerController).fly = true;
            yield return new WaitForSeconds(1);

            //-Assert
            float currentY = player.transform.position.y;
            Assert.Greater(currentY, startY);
        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator NoInput_PlayerFalls()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            float startY = player.transform.position.y;
            PlayerState controller = player.GetComponent<PlayerState>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            //Run the game while receiving no input
            yield return new WaitForSeconds(1);

            //-Assert
            float currentY = player.transform.position.y;
            Assert.Greater(startY, currentY);
        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator NoInput_StateIsFalling()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            float startY = player.transform.position.y;
            PlayerState state = player.GetComponent<PlayerState>();

            //-Act
            //Run the game while receiving simulated input
            GameState.I.SetGameState(GameState.State.Running);
            yield return new WaitForSeconds(1);

            //-Assert
            bool expectedState = false;
            Assert.AreEqual(expectedState, state.IsFlying);
        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator StopInput_PlayerFalls()
        {
            /////-Arrange////
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerMovement controller = player.GetComponent<PlayerMovement>();
            PlayerInput input = player.GetComponent<PlayerInput>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);

            //Simulate receing input for 1 second
            (input.Controller as TestRunnerController).fly = true;
            yield return new WaitForSeconds(1);
            float peakY = player.transform.position.y;

            //Then simulate playing letting go of input
            (input.Controller as TestRunnerController).fly = false;
            yield return new WaitForSeconds(1);

            //-Assert
            float currentY = player.transform.position.y;
            Assert.Greater(peakY, currentY);
        }

        //Note: This tests if the game reacts correctly to input, not if the actual input works
        [UnityTest]
        public IEnumerator StopInput_StateIsFalling()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();
            PlayerInput input = player.GetComponent<PlayerInput>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);

            //Simulate receing input for 1 second
            (input.Controller as TestRunnerController).fly = true;
            yield return new WaitForSeconds(1);

            //Then simulate playing letting go of input
            (input.Controller as TestRunnerController).fly = false;
            yield return new WaitForSeconds(1);

            //-Assert
            bool expectedState = false;
            Assert.AreEqual(expectedState, state.IsFlying);
        }

    }
}
