using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace AlienRunner.Tests
{
    /// <summary>
    /// Represents a function of the eye, providing the ability to find components on GameObjects based on a specified type of input (TInput)
    /// </summary>
    public class EyeFunction<TInput>
    {
        /// <summary>
        /// Represents a way for this eye function to find a component on a GameObject based on the input.
        /// </summary>
        public delegate Component CheckerDelegate(GameObject gameObject, TInput input);  
        /// <summary>
        /// A list of the checker functions this Eye Function uses.
        /// </summary>
        private readonly CheckerDelegate[] checkers;
        /// <summary>
        /// If not null, the parent this Eye Function is contained to checking within.
        /// </summary>
        private readonly GameObject parent;
        public EyeFunction(GameObject parent, params CheckerDelegate[] checkers)
        {
            this.parent = parent;
            this.checkers = checkers;
        }
        /// <summary>
        /// Find all transforms this Eye Function can consider, based on the parent.
        /// </summary>
        private Transform[] FindTransforms()
        {
            return parent == null ? UnityEngine.Object.FindObjectsOfType<Transform>() : parent.GetComponentsInChildren<Transform>();
        }
        /// <summary>
        /// Uses this eye function's checkers to return all components that apply.
        /// </summary>
        private IEnumerable<Component> Finder(TInput input)
        {
            foreach(var transform in FindTransforms())
            {
                foreach(var checker in checkers)
                {
                    var result = checker(transform.gameObject, input);
                    if(result != null)
                    {
                        yield return result;
                        break;
                    }
                }
            }
        }
        /// <summary>
        /// Returns true if the specified screen position is visible on the screen.
        /// </summary>
        private static bool IsVisible(Vector2 screenPos)
        {
            return screenPos.x >= 0 && screenPos.x <= Screen.width && screenPos.y >= 0 && screenPos.y <= Screen.height;
        }

        /// <summary>
        /// Look for a visible, enabled and active game object that matches the input.
        /// </summary>
        public GameObject Look(TInput input)
        {
            return LookAll(input).FirstOrDefault();
        }
        /// <summary>
        /// Look for all visible, enabled and active game objects that match the input.
        /// </summary>
        public IEnumerable<GameObject> LookAll(TInput input)
        {
            foreach (var component in Finder(input))
            {
                //When we "look", the component has to be enabld
                if (component is Behaviour behaviour && !behaviour.enabled)
                {
                    continue;
                }
                //It also has to be active
                if (!component.gameObject.activeInHierarchy || !component.gameObject.activeSelf)
                {
                    continue;
                }
                //And finally it has to be visible
                if (component.GetComponentInParent<Canvas>() != null)
                {
                    if (IsVisible(component.gameObject.CanvasToScreenPos()))
                        yield return component.gameObject;
                }
                else 
                {
                    if (IsVisible(component.gameObject.WorldToScreenPos()))
                        yield return component.gameObject;
                }
            }
        }
        /// <summary>
        /// Find a gameObject that matches the input (even invisible, disabled or inactive)
        /// </summary>
        public GameObject Find(TInput input)
        {
            return FindAll(input).FirstOrDefault();
        }
        /// <summary>
        /// Find all gameObjects that matches the input (even invisible, disabled or inactive)
        /// </summary>
        public IEnumerable<GameObject> FindAll(TInput input)
        {
            return Finder(input).Select(c => c.gameObject);
        }
    }
}
