using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

namespace AlienRunner.Tests
{
    public class EyeInstance 
    {        
        /// <summary>
        /// Finds a TextMeshProUGUI in the gameobject that match the text.
        /// </summary>
        private static Component TextMeshProUGUITextChecker(GameObject gameObject, string text)
        {
            return gameObject.GetComponents<TextMeshProUGUI>().FirstOrDefault(t => t.text == text);
        }
        /// <summary>
        /// Finds a TextMeshPro in the gameobject that match the text.
        /// </summary>
        private static Component TextMeshProTextChecker(GameObject gameObject, string text)
        {
            return gameObject.GetComponents<TextMeshPro>().FirstOrDefault(t => t.text == text);
        }
        /// <summary>
        /// Find a sprite renderer in the gameobject whose sprite name matches the text.
        /// </summary>
        private static Component SpriteRendererSpriteNameChecker(GameObject gameObject, string name)
        {
            return gameObject.GetComponents<SpriteRenderer>().FirstOrDefault(t => t.sprite != null && t.sprite.name == name);
        }
        /// <summary>
        /// Find a UI Images in the gameobject whose sprite name matches the text.
        /// </summary>
        private static Component ImageSpriteNameChecker(GameObject gameObject, string name)
        {
            return gameObject.GetComponents<Image>().FirstOrDefault(t => t.sprite != null && t.sprite.name == name);
        }
        /// <summary>
        /// Returns the gameobject's transform if the name matches the input text.
        /// </summary>
        private static Component TransformNameChecker(GameObject gameObject, string name)
        {
            return gameObject.name == name ? gameObject.transform : null;
        }
        /// <summary>
        /// Returns true if the two colors' RGB values are the same.
        /// </summary>
        private static bool CompareColors(Color a, Color b)
        {
            int aR = Mathf.RoundToInt(a.r * 255);
            int aG = Mathf.RoundToInt(a.g * 255);
            int aB = Mathf.RoundToInt(a.b * 255);
            int bR = Mathf.RoundToInt(b.r * 255);
            int bG = Mathf.RoundToInt(b.g * 255);
            int bB = Mathf.RoundToInt(b.b * 255);
            return aR == bR && aG == bG && aB == bB;
        }
        /// <summary>
        /// Finds a sprite renderer in the GameObject whose color is equal to the input color.
        /// </summary>
        private static Component SpriteRendererColorChecker(GameObject gameObject, Color color)
        {
            return gameObject.GetComponents<SpriteRenderer>().FirstOrDefault(t => CompareColors(t.color, color));
        }
        /// <summary>
        /// Finds an image in the GameObject whose color is equal to the input color.
        /// </summary>
        private static Component ImageColorChecker(GameObject gameObject, Color color)
        {
            return gameObject.GetComponents<Image>().FirstOrDefault(t => CompareColors(t.color, color));
        }
        /// <summary>
        /// Finds a text mesh pro ugui in the GameObject whose color is equal to the input color.
        /// </summary>
        private static Component TextMeshProUGUIColorChecker(GameObject gameObject, Color color)
        {
            return gameObject.GetComponents<TextMeshProUGUI>().FirstOrDefault(t => CompareColors(t.color, color));
        }
        /// <summary>
        /// Finds a text mesh pro in the GameObject whose color is equal to the input color.
        /// </summary>
        private static Component TextMeshProColorChecker(GameObject gameObject, Color color)
        {
            return gameObject.GetComponents<TextMeshPro>().FirstOrDefault(t => CompareColors(t.color, color));
        }

        private EyeFunction<string> text;
        private EyeFunction<string> sprite;
        private EyeFunction<string> name;
        private EyeFunction<Color> color;
        /// <summary>
        /// Use to look for GameObjects based on some text.
        /// </summary>
        public EyeFunction<string> Text => text;
        /// <summary>
        /// Use to look for GameObjects based on a sprite name.
        /// </summary>
        public EyeFunction<string> Sprite => sprite;
        /// <summary>
        /// Use to look for GameObjects based on a GameObject name.
        /// </summary>
        public EyeFunction<string> Name => name;
        /// <summary>
        /// Use to look for GameObjects based on a text, sprite renderer or UI Image color.
        /// </summary>
        public EyeFunction<Color> Color => color;
        /// <summary>
        /// Creates an eye instance which can use various eye functions within the specified GameObject and its children.
        /// Use null to look in all gameobjects in the scene.
        /// </summary>
        public EyeInstance(GameObject parent)
        {
            text = new EyeFunction<string>(parent, TextMeshProTextChecker, TextMeshProUGUITextChecker);
            sprite = new EyeFunction<string>(parent, SpriteRendererSpriteNameChecker, ImageSpriteNameChecker);
            name = new EyeFunction<string>(parent, TransformNameChecker);
            color = new EyeFunction<Color>(parent, TextMeshProColorChecker, TextMeshProUGUIColorChecker, SpriteRendererColorChecker, ImageColorChecker);
        }
    }
}
