using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner.Tests
{
    public static class GameObjectExtensions
    {
        /// <summary>
        /// Convert a canvas rect's center position to screen position
        /// </summary>
        public static Vector2 CanvasToScreenPos(this GameObject ui)
        {
            RectTransform rect = ui.transform as RectTransform;
            if(rect == null)
            {
                return Vector2.zero;
            }
            //The rect's position is actually the pivot's position. By dividing it with the canvas' scale factor, we make it relative to the canvas' actual size.
            Vector2 pivotCanvasSpace = rect.position / rect.GetComponentInParent<Canvas>().scaleFactor;
            //The pivot is normalized, but by multiplying by the rect's size, we get the actual pivot position. We then subtract that from the position, to get the bottom left of the rect.
            Vector2 bottomLeftCanvasSpace = pivotCanvasSpace - rect.pivot * rect.rect.size;
            //Now we add half the size to get the center of the rect in canvas space.
            Vector2 centerCanvasSpace = bottomLeftCanvasSpace + rect.rect.size * 0.5f;
            return CanvasToScreenPos(centerCanvasSpace);
        }
        /// <summary>
        /// Turn a point in canvas space into a screen position
        /// </summary>
        private static Vector2 CanvasToScreenPos(Vector3 pos)
        {
            //Fidn the canvas
            var canvasRect = GameObject.Find("Canvas").GetComponentInChildren<Canvas>().transform as RectTransform;
            //Get a normalized position by dividing with the canvas size.
            var normalized = (pos / canvasRect.rect.size);
            //Screen space's origin is at the top, so we do this to correct
            normalized.y = 1 - normalized.y; 
            //Turn into screen pos by multiplying the normalized position with the screen size.
            var screenPos = normalized * new Vector2(Screen.width, Screen.height);
            return screenPos;
        }
        /// <summary>
        /// Turn a gameobject in the world into a screen position, relative to the main camera
        /// </summary>
        public static Vector2 WorldToScreenPos(this GameObject gameObject)
        {
            var pos = gameObject.transform.position;
            return WorldToScreenPos(pos);
        }
        /// <summary>
        /// Turn a world position into a screen position, relative to the main camera
        /// </summary>
        public static Vector2 WorldToScreenPos(Vector3 pos)
        {
            pos = Camera.main.WorldToScreenPoint(pos);
            pos.y = Screen.height - pos.y;
            return pos;
        }

    }
}
