using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;
namespace AlienRunner.Tests
{
    public class TestTester
    {
        [Test]
        public void EveryTestFirstCallsArrangeReset()
        {
            var assembly = typeof(TestTester).Assembly;
            foreach (var type in assembly.GetTypes())
            {
                //Loop through all types in this assembly
                if (this.GetType() == type || type.IsAbstract)
                {
                    //Skip this class and abstract classes
                    continue;
                }
                //This is the instance of the test class we need to instantiate to probe the tests.
                //It will be instantiated when we need it.
                object instance = null;
                foreach (var method in type.GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.NonPublic))
                {
                    //Loop through all methods.
                    if (method.CustomAttributes.Any(attribute => attribute.AttributeType == typeof(UnityTestAttribute)))
                    {
                        //If this method has the unity test attribute, we assert that its return type is indeed an IEnumerator.
                        Assert.AreEqual(typeof(IEnumerator), method.ReturnType, $"Testing that {type.Name}'s method {method.Name} returns IEnumerator.");
                        if (instance == null)
                        {
                            //Now we instantiate the instance if it hasn't been yet
                            instance = FormatterServices.GetUninitializedObject(type);
                        }
                        //Invoke the method to get the IEnumerator and move to the first element
                        var ienumerator = (IEnumerator)method.Invoke(instance, new object[0]);
                        ienumerator.MoveNext();
                        //We expect that the first yield return is a call to Arrange.Reset, which would return another IEnumerator
                        //So we assert that
                        Assert.IsInstanceOf<IEnumerator>(ienumerator.Current, $"Testing that {type.Name}'s method {method.Name} first calls an IEnumerator method (Arrange.Reset())");
                        //Now we move this returned IEnumerator to the first element, and assert that it's indeed the
                        //-ArrangeYieldInstruction which we know Arrange.Reset returns
                        var arrangeIEnumerator = ienumerator.Current as IEnumerator;
                        arrangeIEnumerator.MoveNext();
                        Assert.IsInstanceOf<ArrangeYieldInstruction>(arrangeIEnumerator.Current, $"Testing that {type.Name}'s method {method.Name} first calls Arrange.Reset()");
                    }
                }
            }
        }
        
    [Test]
    public void GameClassesHaveNoStaticState()
    {
        //Define the assmelbies to check for static state by using a class from each
        var assemblies = new[]
        {
            typeof(AllowedStaticAttribute).Assembly
        };
        //This list will contain error messages about any static state we detect
        List<string> errors = new List<string>();
        foreach (var assembly in assemblies)
        {
            //Loop through each assembly
            foreach (var type in assembly.GetTypes())
            {
                //Loop through each type in the assembly
                if(type.Name.StartsWith("<"))
                {
                    //Ignore weird types that start with <
                    continue;
                }
                foreach (var field in type.GetFields(BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly))
                {
                    //Go through each static field in the type
                    if (field.IsLiteral)
                        continue; //Ignore if it's a constant, that's allowed, since it's immutable
                    if (field.GetCustomAttribute<AllowedStaticAttribute>() != null)
                        continue; //If it explicitly has the AllowedStatic attribute, we allow it.
                    //Add an error for this field.
                    errors.Add($"{type.Name}.{field.Name}:{field.FieldType.Name}");
                }
            }
        }
        if(errors.Count > 0)
        {
            Debug.Log("Static fields detected. Give them [AllowedStatic] (for immutable & readonly fields or other carefully controlled situations).");
        }
        Debug.Log(string.Join("\n", errors));
        //-Assert that there were no errors.
        Assert.AreEqual(0, errors.Count);
    }
    }
}